# USB/UART serial loader for TigerSBC apps from the host

# installation
# cd Desktop/TigerSBC
# assemble this (FPGAburn/appload.py) and use save_hex('FPGAburn/ram.hex') to generate the FPGA RAM initializor
# build your TigerSBC project using iCEcube2
# burn your iCE40-HX8K board SPI configuration flash using Diamond
# when successful, no further iCEcube2 or Diamond sessions are required
# nrst_i is now used to start new serial loads of TigerSBC apps from the host

# after assembly of TigerSBC apps, use save_app('apps/yourapp.app') to generate object files
# use your serial terminal emulator to send .app files raw to the board

  # start execution at RAM[0] after reset
  ljmp(fwd('binit')) # skip the boilerplate

# define some data

lbl('signon_msg'); ascii('\r\nTiger is ready to load...')
lbl('done_msg'); ascii('done\r\n')
lbl('appload_dest'); data([0]) # dummy adr

# define some Tigerasm procedures

lbl('rx') # -> d
# await zero-extended 8 lsbs from USB/UART
    lld(rx_status); ljz(rx)
  lld(uart); ret()

lbl('tx') # d ->
# transmit 8 lsbs of d to USB/UART
    lld(tx_status); ljz(tx) # await tx ready
  lst(uart); ret()

lbl('tx_ascii') # adr ->
# transmit a zero-terminated ascii string to USB/UART
    tt(); ld(); tt(); ljz(fwd('tx_ascii_exit')) # -> adr ascii
      lcall(tx) # -> adr
      inc(); ljmp(tx_ascii) # -> adr'
lbl('tx_ascii_exit')
  drop(); drop(); ret()

lbl('rx2x6') # -> d
# receive a pair of USB/UART data bytes within 0..63, ls byte first, pack
  lcall(rx); lcall(rx) # -> ls6 ms6
  # shift ms6 left 6
#  shl(); shl(); shl(); shl(); shl(); shl() # use the cpu
  lst(mfa); lit(64); lst(mfb); lld(prod11) # use the macc
  # merge with ls6
  bxor(); ret()

# get the new app

lbl('binit') # from reset
  lit(signon_msg); lcall(tx_ascii)
  lit(fwd('app_start')); lst(appload_dest)
  # get app_end
  lcall(rx2x6) # -> app_end
lbl('appload_payload')
    tt(); lld(appload_dest); bxor(); ljz(fwd('appload_done')) # if app_end not reached, get another pair
      lcall(rx2x6) # -> app_end d
      lld(appload_dest); st() # -> app_end
      lld(appload_dest); inc(); lst(appload_dest) # bump dest adr
      ljmp(appload_payload)
lbl('appload_done')
  lit(done_msg); lcall(tx_ascii)
# drop through to the downloaded app, abandon d:, p: is empty
lbl('app_start') # must be on the last line of appload.py
