// signed multiplier/accumulator

module macc (
  input clk_i,
  input sel_i,       // 8-aligned
  input [2:0] adr_i, // subselect
  input storex_i,
  input [11:0] dat_i,
  input loadex_i,
  output [11:0] dat_o
);

reg [11:0] fa, fb; // multiplier factors
wire [23:0] product;

smul smul1 (
  .factor_a_i(fa),
  .factor_b_i(fb),
  .product_o(product)
);

reg [27:0] sop; // sum of products

// write operations
always @(posedge clk_i)
  if (sel_i & storex_i)
    case(adr_i[1:0]) // aliased at adr+4
      0: sop <= {16'b0, dat_i}; // rnd_sop
      1: fa <= dat_i; // mfa
      2: fb <= dat_i; // mfb
      3: sop <= sop + {{4{product[23]}}, product}; // acc_sop, ignore dat_i
    endcase

// read operations
reg [11:0] dat_o;
always @*
  if (sel_i & loadex_i)
    case(adr_i)
      0: dat_o = product[11:0]; // prod11
      1: dat_o = product[23:12]; // prod23
      2: dat_o = sop[22:11]; // norm1
      3: dat_o = sop[23:12]; // norm2
      4: dat_o = sop[24:13]; // norm4
      5: dat_o = sop[25:14]; // norm8
      6: dat_o = sop[26:15]; // norm16
      7: dat_o = sop[27:16]; // norm32
    endcase
  else dat_o = 0; // ORable

endmodule
