// 12-bit deluxe parallel io

module pio (
  // CPU interface
  input clk_i,
  input rst_i,
  input sel_i,       // 4-aligned
  input [1:0] adr_i, // subselect
  input storex_i,
  input [11:0] dat_i,
  input loadex_i,
  output [11:0] dat_o,
  // outside world interface
  input [11:0] pi_i,
  output [11:0] po_o,
  output led_o
);

// write operations
reg [11:0] po_o_;
assign po_o = po_o_;
assign led_o = po_o_[0];
always @(posedge clk_i)
  if (rst_i) po_o_ <= 0;
  else if (sel_i & storex_i)
    case (adr_i)
      0: po_o_ <= dat_i;         // po
      1: po_o_ <= ~dat_i & po_o_; // poc
      2: po_o_ <= dat_i | po_o_; // pos
      3: po_o_ <= dat_i ^ po_o_; // poi
    endcase

// input synchronizer
reg [11:0] pi;
always @(posedge clk_i) pi <= pi_i;

// read operations
reg dat_o;
always @*
  if (sel_i & loadex_i)
    case (adr_i[0]) // alias +2
      0: dat_o = po_o_; // po
      1: dat_o = pi;    // pi
    endcase
  else dat_o = 0; // ORable

endmodule
