// 48-bit free-running timer

module timer (
  input clk_i,
  input rst_i,
  input sel_i,       // 8-aligned
  input [2:0] adr_i, // subselect
  input loadex_i,
  output [11:0] dat_o
);

reg [47:0] count;
always @(posedge clk_i)
  if (rst_i) count <= 0;
  else count <= count + 1;

reg [47:12] upper;
always @(posedge clk_i)
  if (sel_i & (adr_i == 3'b100) & loadex_i) upper <= count[47:12];

// read operations
reg [11:0] dat_o;
always @*
  if (sel_i & loadex_i)
    case(adr_i)
      0: dat_o = count[11:0];  // t0
      1: dat_o = count[23:12]; // t1
      2: dat_o = count[35:24]; // t2
      3: dat_o = count[47:36]; // t3
      4: dat_o = count[11:0];  // tl0
      5: dat_o = upper[23:12]; // tl1
      6: dat_o = upper[35:24]; // tl2
      7: dat_o = upper[47:36]; // tl3
    endcase
  else dat_o = 0; // ORable

endmodule
