// 115200,8,n,1 UART

module uart (
  // CPU interface
  input clk_i,
  input rst_i,
  input sel_i,       // 4-aligned
  input [1:0] adr_i, // subselect
  input storex_i,
  input [7:0] dat_i,
  input loadex_i,
  output [7:0] dat_o,
  // uart interface
  input rxd_i,
  output txd_o
);

wire tx_we = sel_i & (adr_i == 2'b10) & storex_i; // uart
wire rx_re = sel_i & (adr_i == 2'b10) & loadex_i; // uart
wire tx_rdy, rx_rdy;
wire [7:0] rx_dat;

atx atx1 (
  .clk_i(clk_i),
  .rst_i(rst_i),
  .we_i(tx_we),
  .dat_i(dat_i),
  .txd_o(txd_o),
  .ready_o(tx_rdy)
);

arx arx1 (
  .clk_i(clk_i),
  .rst_i(rst_i),
  .rxd_i(rxd_i),
  .rst_ready_i(rx_re),
  .ready_o(rx_rdy),
  .dat_o(rx_dat)
);

// read operations
reg dat_o;
always @*
  if (sel_i & loadex_i)
    case (adr_i)
      0: dat_o = {7'b0, rx_rdy}; // arx_status
      1: dat_o = {7'b0, tx_rdy}; // atx_status 
      2: dat_o = rx_dat; // uart
      3: dat_o = 0;
    endcase
  else dat_o = 0; // ORable

endmodule
