include('FPGAburn/appload.py') # must be on the first line

# generate audio sawtooth waves on po_o at fs
# tested with an 8-bit 10k r2r ladder on po_o[11:4]
# a 3V3 square wave is also available on po_o[11]

  ljmp(fwd('init'))

# constants

# fout = fs*phinc/2**24
# phinc = fout*2**24/fs
defnum('doit_clks', 31) # calibrated per analysis of doit loop below
defnum('fs', fclk/doit_clks)
print('fs is', fs, 'Hz', 'pass', passn)

defnum('k1Hz', 2**24/fs)
def12('k1Hz0', int(k1Hz) & ALLBITS)
def12('k1Hz1', int(k1Hz) >> 12)

defnum('k10Hz', k1Hz*10)
def12('k10Hz0', int(k10Hz) & ALLBITS)
def12('k10Hz1', int(k10Hz) >> 12)

defnum('k100Hz', k1Hz*100)
def12('k100Hz0', int(k100Hz) & ALLBITS)
def12('k100Hz1', int(k100Hz) >> 12)

defnum('k1kHz', k1Hz*1000)
def12('k1kHz0', int(k1kHz) & ALLBITS)
def12('k1kHz1', int(k1kHz) >> 12)

defnum('k440Hz', k1Hz*440)
def12('k440Hz0', int(k440Hz) & ALLBITS)
def12('k440Hz1', int(k440Hz) >> 12)

# little-endian 24-bit variables

lbl('phacc'); data([0,0]) # phase accumulator
lbl('phinc'); data([0,0]) # phase increment

# keystroke command procedures

lbl('got_upHz') # keystroke ->
# up 1 Hz
  drop() # ->
  lit(k1Hz0); lld(phinc); adx(); ladd(k1Hz1); lld(phinc+1); add()
  lst(phinc+1); lst(phinc); ret()

lbl('got_downHz') # keystroke ->
# down 1 Hz
  drop() # ->
  lld(phinc); lit(k1Hz0); neg(); adx(); lld(phinc+1); add(); lit(k1Hz1); inv(); add()
  lst(phinc+1); lst(phinc); ret()

lbl('got_upoctave') # keystroke ->
# up 1 octave
  drop() # ->
  lld(phinc); tt(); adx(); lld(phinc+1); tt(); add(); add()
  lst(phinc+1); lst(phinc); ret()

lbl('got_downoctave') # keystroke ->
# down 1 octave
  drop() # ->
  lld(phinc); asr(); lband(MAXPOS); lld(phinc+1) # -> phinc0' phinc1
  tt(); asr(); lst(phinc+1)
  lband(1); ljz(fwd('got_downoctave_exit')) # -> phinc0'
    lbxor(SIGNBIT)
lbl('got_downoctave_exit')
  lst(phinc); ret()

lbl('got_1Hz') # keystroke ->
# generate 1 Hz
  drop() # ->
  lit(k1Hz0); lst(phinc); lit(k1Hz1); lst(phinc+1); ret()

lbl('got_10Hz') # keystroke ->
# generate 10 Hz
  drop() # ->
  lit(k10Hz0); lst(phinc); lit(k10Hz1); lst(phinc+1); ret()

lbl('got_100Hz') # keystroke ->
# generate 100 Hz
  drop() # ->
  lit(k100Hz0); lst(phinc); lit(k100Hz1); lst(phinc+1); ret()

lbl('got_1kHz') # keystroke ->
# generate 1 kHz
  drop() # ->
  lit(k1kHz0); lst(phinc); lit(k1kHz1); lst(phinc+1); ret()

lbl('got_A440') # keystroke ->
# generate 440 Hz
  drop() # ->
  lit(k440Hz0); lst(phinc); lit(k440Hz1); lst(phinc+1); ret()

lbl('command') # ->
# respond to USB/UART keystrokes
  lld(rx_status); ljz(fwd('no_command'))
    lld(uart) # -> keystroke
    tt(); lcall(tx) # echo
    # resolve defined keystroke commands
    tt(); lbxor(ord('H')); ljz(got_upHz)
    tt(); lbxor(ord('h')); ljz(got_downHz)
    tt(); lbxor(ord('O')); ljz(got_upoctave)
    tt(); lbxor(ord('o')); ljz(got_downoctave)
    tt(); lbxor(ord('0')); ljz(got_1Hz)
    tt(); lbxor(ord('1')); ljz(got_10Hz)
    tt(); lbxor(ord('2')); ljz(got_100Hz)
    tt(); lbxor(ord('3')); ljz(got_1kHz)
    tt(); lbxor(ord('a')); ljz(got_A440)
    drop() # -> # none of the above, drop keystroke
lbl('no_command')
  ret()

# the app

lbl('init')
  tt(); lcall(got_A440) # set default Hz with bogus keystroke arg
lbl('doit') # main loop troika'd for timing analysis
    lit(command); call()           # nominal 8 clks, user control of phinc
    lit(phacc); ld(); align()      # 3 clks (11), -> phacc0 # optimize prefetch
    lit(phinc); ld(); adx()        # 3 clks (14), -> phacc0' carry
    lit(phacc+1); ld(); add()      # 3 clks (17), -> phacc0' phacc1+carry
    lit(phinc+1); ld(); add()      # 3 clks (20), -> phacc0' phacc1'
    tt(); lit(po); st()            # 4 clks (24)
    lit(phacc+1); st(); lit(phacc) # 4 clks (28)
    st(); lit(doit); jmp()         # 3 clks (31)

lbl('app_end') # must be on the last line
