# Fast Fourier Transform
# in-place radix-2 DIT, bit-reversed input order, normal output order
# requires prior def12s of passes and points

# in-place data
lbl('real'); data([0 for n in range(points)])
lbl('imag'); data([0 for n in range(points)])

# trig luts
lbl('cos'); data([int(MAXPOS*math.cos(n*math.tau/points)) for n in range(points)])
lbl('sin'); data([int(MAXPOS*math.sin(n*math.tau/points)) for n in range(points)])

# bit-reverse lut, initialized at run-time by the set_brev procedure below
lbl('brev'); data([0 for n in range(points)]) # dummy values for now

# fft simple variables
lbl('phase'); data([0])  # trig lookup tables index
lbl('phinc'); data([0])  # trig phase increment
lbl('offset'); data([0]) # re low and high indices
lbl('low'); data([0])    # data butterfly index
lbl('high'); data([0])   # data butterfly index
lbl('treal'); data([0])  # scratchpad
lbl('timag'); data([0])  # scratchpad

lbl('fft') # ->
# in-place fft of real and imag arrays
  lit(1); lst(offset)
  lit(points >> 1); lst(phinc)
  lit(0) # -> pass
lbl('pass_loop')
    tt(); lbxor(passes); ljz(fwd('passes_done'))
      lit(0) # -> pass group
lbl('group_loop')
        tt(); lbxor(points); ljz(fwd('groups_done'))
          tt(); lst(low)
          lit(0); tt(); lst(phase) # -> pass group butterfly
lbl('butterfly_loop')
            tt(); lld(offset); bxor(); ljz(fwd('butterflies_done'))
              lld(low); lld(offset); add(); lst(high)
              # treal = (cos[phase]*real[high] + sin[phase]*imag[high])/2
              lit(1 << 11); lst(rnd_sop)
              lld(phase); lald(cos); lst(mfa)
              lld(high); lald(real); lst(mfb)
              tt(); lst(acc_sop)
              lld(phase); lald(sin); lst(mfa)
              lld(high); lald(imag); lst(mfb)
              tt(); lst(acc_sop)
              lld(norm2); lst(treal)
              # timag = (cos[phase]*imag[high] - sine[phase]*real[high])/2
              lit(1 << 11); lst(rnd_sop)
              lld(phase); lald(cos); lst(mfa)
              lld(high); lald(imag); lst(mfb)
              tt(); lst(acc_sop)
              lld(phase); lald(sin); neg(); lst(mfa)
              lld(high); lald(real); lst(mfb)
              tt(); lst(acc_sop)
              lld(norm2); lst(timag)
              # real[high] = real[low]/2 - treal
              # real[low] = real[low]/2 + treal
              lld(low); lald(real); asr()
              tt(); lld(treal); sub(); lld(high); last(real)
              lld(treal); add(); lld(low); last(real)
              # imag[high] = imag[low]/2 - timag
              # imag[low] = imag[low]/2 + timag
              lld(low); lald(imag); asr()
              tt(); lld(timag); sub(); lld(high); last(imag)
              lld(timag); add(); lld(low); last(imag)
              #
              lld(phase); lld(phinc); add(); lst(phase)
              lld(low); inc(); lst(low)
              inc() # -> pass group butterfly'
              ljmp(butterfly_loop)
lbl('butterflies_done')
          drop() # -> pass group
          lld(offset); shl(); add() # -> pass group'
          ljmp(group_loop)
lbl('groups_done')
      drop() # -> pass
      lld(offset); shl(); lst(offset)
      lld(phinc); asr(); lst(phinc)
      inc() # -> pass'
      ljmp(pass_loop)
lbl('passes_done')
  drop(); ret() # ->


lbl('bitrev_imask'); data([0])
lbl('bitrev_omask'); data([0])
lbl('bitrev_acc'); data([0])
lbl('bitrev_d'); data([0])
#
lbl('bitrev') # d -> d'
# bit reversal
  lst(bitrev_d) # ->
  lit(1); lst(bitrev_imask)
  lit(points >> 1); lst(bitrev_omask)
  lit(0); tt(); lst(bitrev_acc) # -> index
lbl('bitrev_loop')
    tt(); lbxor(passes); ljz(fwd('bitrev_done'))
      lld(bitrev_d); lld(bitrev_imask); band(); ljz(fwd('bitrev_zero'))
        lld(bitrev_acc); lld(bitrev_omask); bxor(); lst(bitrev_acc)
lbl('bitrev_zero')
      lld(bitrev_imask); shl(); lst(bitrev_imask)
      lld(bitrev_omask); asr(); lst(bitrev_omask)
      inc() # -> index'
      ljmp(bitrev_loop)
lbl('bitrev_done')
  drop(); lld(bitrev_acc); ret() # -> d'


lbl('set_brev_adr'); data([0])
#
lbl('set_brev') # ->
# set the brev lut
  lit(0) # -> index
lbl('set_brev_loop')
    tt(); lbxor(points); ljz(fwd('set_brev_done'))
      tt(); ladd(brev); lst(set_brev_adr)
      tt(); lcall(bitrev) # -> index xedni
      lld(set_brev_adr); st() # -> index
      inc() # -> index'
      ljmp(set_brev_loop)
lbl('set_brev_done')
  drop(); ret() # ->
