# FM25V20A FRAM SPI interface
# tie pin 7 DNU to VDD to avoid crosstalk from pin 6 SCK!
# requires lib/spi.py

def12('fm25v20a_speed', 0 << 3) # 18 MHz sck, upper 5-bit field for lst(spi_ctl)
def12('fm25v20a_wren', 6) # opcode
def12('fm25v20a_write', 2) # opcode
def12('fm25v20a_read', 3) # opcode

# Tiger addresses 12-bit words, but the fram addresses 8-bit bytes.
# Therefore little-endian 24-bit transfers (2 words, 3 bytes, 1 troika) are performed between fram and Tiger memory.

lbl('fr1') # -> d7:0
# read 1 byte from active fram
  lit(0); lst(spi); lcall(spi_wait) # tx dummy
  lld(spi); ret()

lbl('fr3') # -> d7:0 d15:8 d23:16
# read 3 bytes from active fram
  lcall(fr1); lcall(fr1); ljmp(fr1)

lbl('fw1') # d7:0 ->
# write 1 byte to active fram
  lst(spi); ljmp(spi_wait)

lbl('fw3') # d23:16 d15:8 d7:0 ->
# write 3 bytes to active fram
  lcall(fw1); lcall(fw1); ljmp(fw1)

lbl('pupd'); data([0]) # re pack/unpack

lbl('pack3_2') # d7:0 d15:8 d23:16 -> d11:0 d23:12
# convert 3 fram read bytes to 2 Tiger words
  lst(mfa); lit(0x10); lst(mfb) # -> d7:0 d15:8
  lld(prod11); lst(pupd)
  lst(mfa); lit(0x100); lst(mfb) # -> d7:0
  lld(prod11); bxor() # -> d11:0
  lld(prod23); lld(pupd); bxor() # -> d11:0 d23:12
  ret()

lbl('unpack2_3') # d11:0 d23:12 -> d23:16 d15:8 d7:0
# convert 2 Tiger words to 3 fram write bytes
  lst(mfa); lst(pupd) # ->
  lit(0x100); lst(mfb); lld(prod23) # -> d23:16
  lit(0x10); lst(mfb); lld(prod11) # -> d23:16 d15:8'
  lld(pupd); lst(mfa); lld(prod23); lband(0xf); bxor() # -> d23:16 d15:8
  lld(pupd) # -> d23:16 d15:8 d7:0
  ret()

lbl('ram_adr'); data([0]) # src/dest
lbl('troikas'); data([0]) # loop control

lbl('wren') # spi_select' ->
  lst(spi_ctl); lit(fm25v20a_wren); lcall(fw1)
  lit(0); lst(spi_ctl); ret()

lbl('fram_write') # ram_adr troikas fram_a7:0 fram_a15:8 fram_a17:16 spi_select ->
# write troikas from ram_adr to fram
  lbxor(fm25v20a_speed); tt(); lcall(wren); lst(spi_ctl) # -> ram_adr troikas fram_a7:0 fram_a15:8 fram_a17:16
  # fram write seek
  lit(fm25v20a_write); lcall(fw1)
  lcall(fw3) # -> ram_adr troikas
  # set loop vars
  neg(); lst(troikas) # -> ram_adr
  lst(ram_adr) # ->
lbl('fram_write_loop')
  lld(troikas); ljz(fwd('fram_write_done'))
    lld(ram_adr); ld() # -> d11:0
    lld(ram_adr); inc(); ld() # -> d11:0 d23:12
    lcall(unpack2_3); lcall(fw3) # ->
    lld(ram_adr); inc(); inc(); lst(ram_adr)
    lld(troikas); inc(); lst(troikas)
    ljmp(fram_write_loop)
lbl('fram_write_done')
  lit(0); lst(spi_ctl); ret()

lbl('fram_read') # ram_adr troikas fram_a7:0 fram_a15:8 fram_a17:16 spi_select ->
# read troikas from fram to ram_adr
  lbxor(fm25v20a_speed); lst(spi_ctl) # -> ram_adr troikas fram_a7:0 fram_a15:8 fram_a17:16
  # fram read seek
  lit(fm25v20a_read); lcall(fw1)
  lcall(fw3) # -> ram_adr troikas
  # set loop vars
  neg(); lst(troikas) # -> ram_adr
  lst(ram_adr) # ->
lbl('fram_read_loop')
  lld(troikas); ljz(fwd('fram_read_done'))
    lcall(fr3) # -> d7:0 d15:8 d23:16
    lcall(pack3_2) # -> d11:0 d23:12
    lld(ram_adr); inc(); st() # -> d11:0
    lld(ram_adr); st() # ->
    lld(ram_adr); inc(); inc(); lst(ram_adr)
    lld(troikas); inc(); lst(troikas)
    ljmp(fram_read_loop)
lbl('fram_read_done')
  lit(0); lst(spi_ctl); ret()
