module macc_tb; // no external inputs or outputs

// inputs
reg clk_i;
reg sel_i;
reg [2:0] adr_i;
reg storex_i;
reg [11:0] dat_i;
reg loadex_i;

// outputs
wire [11:0] dat_o;

// module under test
macc mut (
  .clk_i(clk_i),
  .sel_i(sel_i),
  .adr_i(adr_i),
  .storex_i(storex_i),
  .dat_i(dat_i),
  .loadex_i(loadex_i),
  .dat_o(dat_o)
);

parameter PERIOD = 2; // half-clocks

// clk_i generator
always begin
  clk_i = 0;
  #(PERIOD/2);
  clk_i = 1;
  #(PERIOD/2);
end

// input stimulus
initial begin
  $display("idle, expect 0 on dat_o"); // 0
  sel_i = 0; storex_i = 0; loadex_i = 0; adr_i = 3'h0; dat_i = 12'h000; #(PERIOD*1);

  $display("write 0x800 to rnd_sop"); // 2
  sel_i = 1; storex_i = 1; loadex_i = 0; adr_i = 3'h0; dat_i = 12'h800; #(PERIOD*1);

  $display("read sop21, expect 2"); // 4
  sel_i = 1; storex_i = 0; loadex_i = 1; adr_i = 3'h2; dat_i = 12'h800; #(PERIOD*1);

  $display("read sop22, expect 1"); // 6
  sel_i = 1; storex_i = 0; loadex_i = 1; adr_i = 3'h3; dat_i = 12'h800; #(PERIOD*1);

  $display("read sop23, expect 0"); // 8
  sel_i = 1; storex_i = 0; loadex_i = 1; adr_i = 3'h4; dat_i = 12'h800; #(PERIOD*1);

  $display("write 0x800 to mfa and mfb, singular product of 0x400000"); // 10
  sel_i = 1; storex_i = 1; loadex_i = 0; adr_i = 3'h1; dat_i = 12'h800; #(PERIOD*1);
  sel_i = 1; storex_i = 1; loadex_i = 0; adr_i = 3'h2; dat_i = 12'h800; #(PERIOD*1);

  $display("read prod11, expect 0"); // 14
  sel_i = 1; storex_i = 0; loadex_i = 1; adr_i = 3'h0; dat_i = 12'h800; #(PERIOD*1);

  $display("read prod23, expect 0x400"); // 16
  sel_i = 1; storex_i = 0; loadex_i = 1; adr_i = 3'h1; dat_i = 12'h800; #(PERIOD*1);

  $display("accumulate1, read sop23, expect 0x400"); // 18
  sel_i = 1; storex_i = 1; loadex_i = 0; adr_i = 3'h3; dat_i = 12'h800; #(PERIOD*1);
  sel_i = 1; storex_i = 0; loadex_i = 1; adr_i = 3'h4; dat_i = 12'h800; #(PERIOD*1);

  $display("accumulate2, read sop24, expect 0x400"); // 22
  sel_i = 1; storex_i = 1; loadex_i = 0; adr_i = 3'h3; dat_i = 12'h800; #(PERIOD*1);
  sel_i = 1; storex_i = 0; loadex_i = 1; adr_i = 3'h5; dat_i = 12'h800; #(PERIOD*1);

  $display("accumulate3..4, read sop25, expect 0x400"); // 26
  sel_i = 1; storex_i = 1; loadex_i = 0; adr_i = 3'h3; dat_i = 12'h800; #(PERIOD*2);
  sel_i = 1; storex_i = 0; loadex_i = 1; adr_i = 3'h6; dat_i = 12'h800; #(PERIOD*1);

  $display("accumulate5..8, read sop26, expect 0x400"); // 32
  sel_i = 1; storex_i = 1; loadex_i = 0; adr_i = 3'h3; dat_i = 12'h800; #(PERIOD*4);
  sel_i = 1; storex_i = 0; loadex_i = 1; adr_i = 3'h7; dat_i = 12'h800; #(PERIOD*1);

  $display("end time is%d cp/2", $stime);
  $finish;
end

// Icarus Verilog event output
initial
  $monitor(
    "t%t cp/2|sel_i:%h|adr_i:%h|storex_i:%h|dat_i:%h|loadex_i:%h|dat_o:%h",
    $time,
    sel_i,
    adr_i,
    storex_i,
    dat_i,
    loadex_i,
    dat_o
  );

endmodule