module spi_tb; // no external inputs or outputs

// inputs
  reg clk_i;
  reg rst_i;
  reg sel_i;
  reg adr_i;
  reg storex_i;
  reg [7:0] dat_i;
  reg loadex_i;
  reg miso_i;

// outputs
  wire [7:0] dat_o;
  wire sck_o;
  wire mosi_o;
  wire [7:1] ncs_o;

// module under test
spi mut (
  .clk_i(clk_i),
  .rst_i(rst_i),
  .sel_i(sel_i),
  .adr_i(adr_i),
  .storex_i(storex_i),
  .dat_i(dat_i),
  .loadex_i(loadex_i),
  .dat_o(dat_o),
  .miso_i(miso_i),
  .sck_o(sck_o),
  .mosi_o(mosi_o),
  .ncs_o(ncs_o)
);

parameter PERIOD = 2; // half-clocks

// clk_i generator
always begin
  clk_i = 0;
  #(PERIOD/2);
  clk_i = 1;
  #(PERIOD/2);
end

// input stimulus
initial begin
  $display("reset"); // 0
  rst_i = 1; sel_i = 0; storex_i = 0; loadex_i = 0; miso_i = 1;
  #(PERIOD*1);

  $display("set sel_i; adr_i, storex_i, and loadex_i will control data transfers"); // 2
  rst_i = 0; sel_i = 1;
  #(PERIOD*1);

  $display("select ncs1, fsck = fclk/2"); // 4
  adr_i = 0; storex_i = 1; dat_i = 1;
  #(PERIOD*1);
  storex_i = 0; // 6
  #(PERIOD*1);

  $display("tx 0xa5, monitor status, expect rx 0xf0"); // 8
  adr_i = 1; storex_i = 1; dat_i = 8'ha5;
  #(PERIOD*1);
  adr_i = 0; storex_i = 0; loadex_i = 1; // 10
  #15;
  miso_i = 0; // 25
  #25;
  adr_i = 1; // 50
  #(PERIOD*1);

  $display("select ncs2, fsck = fclk/64"); // 52
  adr_i = 0; storex_i = 1; dat_i = 8'b11111010; loadex_i = 0;
  #(PERIOD*1);
  storex_i = 0; // 54
  #(PERIOD*1);

  $display("tx 0x5a, monitor raw miso expecting 0xff, expect status 1"); // 56
  adr_i = 1; storex_i = 1; dat_i = 8'h5a; miso_i = 1;
  #(PERIOD*1);
  storex_i = 0; loadex_i = 1; // 58
  #(PERIOD*1000);
  adr_i = 0; // 2058
  #(PERIOD*1);

  $display("end time is%d cp/2", $stime);
  $finish;
end

// Icarus Verilog event output
initial
  $monitor(
    "t%t cp/2|sel_i:%b|adr_i:%h|storex_i:%b|dat_i:%h|miso_i:%b|sck_o:%b|mosi_o:%b|ncs_o:%b|loadex_i:%b|dat_o:%h",
    $time,
    sel_i,
    adr_i,
    storex_i,
    dat_i,
    miso_i,
    sck_o,
    mosi_o,
    ncs_o,
    loadex_i,
    dat_o
  );

endmodule